#!/usr/bin/tcl -f

###############################################
#  This script converts Phylip distance       #
#  matrix file into "three column (binary)"   #
#  matrix file format                         #
#                                             #
# Copyright(C) Alexander Kozik, UC Davis, CA  #
#              December 28 2001               #
###############################################

proc Build_Three_Line_Matrix { f_out } {

    global temporary_data_file list_of_elements
    set temporary_data [open "$temporary_data_file" "r"]
    set k 1

    while {[gets $temporary_data  current_line] >= 0}  {
	set current_element_A [lindex [split $current_line " "] 0]
	set current_data_list [split $current_line " "]
	Process_Current_Line $current_element_A $current_data_list $k $f_out
	incr k
    }

    close $temporary_data
}

proc Process_Current_Line { current_element_A current_data_list k f_out} {

    global list_of_elements
    global identity_cutoff

    set m 1

    while {[expr $k - $m] >= 0} {

	set current_element_B [lindex [split $list_of_elements] [expr $m -1]]
	set current_data_value [lindex $current_data_list [expr $m ]]
	set current_similarity [expr 1 - $current_data_value]

	if {$current_element_A != $current_element_B && $current_similarity >= $identity_cutoff} {
	    puts $f_out "$current_element_A\t$current_element_B\t$current_similarity"
	}
	incr m
    }

}

# MAIN BODY #

# standard input/output dialog #

puts -nonewline "Enter the SOURCE file name: "
flush stdout

set sourcefile [gets stdin]

while {[catch {set f_in [open "$sourcefile"  "r"]}] == 1} {
        puts stderr "Error opening $sourcefile"
        puts -nonewline "Please re-enter the SOURCE file name: "
        flush stdout
        set sourcefile [gets stdin]
}

puts -nonewline "Enter the DESTINATION file name: "
flush stdout

set destfile [gets stdin]

set f_out [open "$destfile"  "w"]

global identity_cutoff

puts -nonewline "Enter Identity cutoff value (0.4): "
flush stdout

set identity_cutoff [gets stdin]

# end of input/output dialog #

global temporary_data_file list_of_elements

set lineNumber 0
set first_line_to_process 1
set new_line ""
set i 1
set list_of_elements {}

set temporary_data_file "temporary_matrix.txt"
set temporary_data [open "$temporary_data_file" "w"]

while {[gets $f_in  current_line] >= 0}  {

        if {$lineNumber >= $first_line_to_process} {
	    regsub -all {\s+} $current_line " " current_line
	    regsub {\s$} $current_line "" current_line
	    set new_element [regexp {^\s} $current_line]
	    if {$new_element == 1} {
		puts -nonewline $temporary_data $current_line
	    }
	    if {$new_element != 1} {
		set current_element($i) [lindex [split $current_line " "] 0]
		puts "$current_element($i)\t$i"
		lappend list_of_elements $current_element($i)
		puts -nonewline $temporary_data "$new_line$current_line"
		incr i
		set new_line "\n"
	    }
	}
	incr lineNumber
    }
close $f_in
puts -nonewline $temporary_data "\n"
close $temporary_data

Build_Three_Line_Matrix $f_out

close $f_out
